#!/usr/bin/newlisp


(define (FILTERS:upper str)
  "Translates str to all upper-case characters."
  (upper-case str))


(define (FILTERS:lower str)
  "Translates str to all lower-case characters."
  (lower-case str))


(define (FILTERS:title str)
  "A better title-case function that takes into account basic rules of
  capitalization."
  (let ((do-not-capitalize
         '("the" "a" "an" "in" "and" "as" "but" "if" "or" "nor"
           "at" "by" "for" "in" "of" "off" "on" "out" "to" "up")))
    ;; capitalize all words
    (replace {\b(\w)} str (upper-case $1) 1)

    ;; uncapitalize prohibited words
    (dolist (w do-not-capitalize)
      (replace (string {\b} w {\b}) str w 1))

    ;; capitalize first word and last word
    (nth-set (str 0) (upper-case (str 0)))
    (replace {\b(\w)\S+?$} str (title-case $0) 1)

    str))


(define (FILTERS:date-format str frmt)
  "Formats timestamp value str (string of integer) according to formatting
  rules defined for `date`, using frmt.  Usage:
  
  {{ timestampe|date:\"%m/%d/%y\" }}
  
  (note that the double-quotes do not need to be escaped in the template
  itself.)"
  (date (int str) 0 frmt))


(define (filter? str)
  (and (context? FILTERS str)
       (lambda? (context FILTERS str))))