#!/usr/bin/env newlisp


;;; Note: POST data can only be read once, after which it becomes unavailable
;;; to future parts of the program.  Should the default cgi.lsp module precede
;;; this module, POST data will be unavailable through the Request class.  If
;;; cgi.lsp is loaded after this module, POST data is unavailable to it.

;;; This module does not include output functions, including setting cookies.
;;; That will be part of the Response class.  The Response class may also
;;; include a session framework; any session data will be available through this
;;; class (once designed).


(constant '*max-post-length* 4096)
(global '*max-post-length*)


(context 'Request)


(set '_cgi-keys '("REDIRECT_STATUS" "HTTP_HOST" "HTTP_USER_AGENT" "HTTP_ACCEPT"
  "HTTP_ACCEPT_LANGUAGE" "HTTP_ACCEPT_ENCODING" "HTTP_ACCEPT_CHARSET"
  "HTTP_KEEP_ALIVE" "HTTP_CONNECTION" "HTTP_COOKIE" "HTTP_CACHE_CONTROL" "PATH"
  "SERVER_SIGNATURE" "SERVER_SOFTWARE" "SERVER_NAME" "SERVER_ADDR" "SERVER_PORT"
  "REMOTE_ADDR" "DOCUMENT_ROOT" "SERVER_ADMIN" "SCRIPT_FILENAME" "REMOTE_PORT"
  "REDIRECT_URL" "GATEWAY_INTERFACE" "SERVER_PROTOCOL" "REQUEST_METHOD"
  "QUERY_STRING" "REQUEST_URI" "SCRIPT_NAME" "PATH_INFO" "PATH_TRANSLATED"))


;; set cleaned CGI environment parameters
(set '_cgi-env (map (fn (key) (list key (trim (string (env key))))) _cgi-keys))


;; set default method
(set '_method "get")


(define (parse-query query-string)
  (let ((params '()))
    (dolist (element (parse query-string "&"))
      (let ((pair (parse element "=")))
        (if (= 1 (length pair)) (push nil pair -1))
        (push pair params -1)))
    params))


;; deal with GET params from QUERY_STRING
(set '_get (parse-query (lookup "QUERY_STRING" _cgi-env)))


;; deal with POST params from stdin data
(read-buffer (device) 'post-data *max-post-length*)
(if post-data
  (begin
    (set '_method "post")
    (set '_post (parse-query post-data)))
  (set '_post '()))


;; deal with HTTP_COOKIE data
(set '_cookies '())
(dolist (element (parse (lookup "HTTP_COOKIE" _cgi-env) ";"))
  (let ((pair (parse element "=")))
    (push pair _cookies -1)))


;; store segment information
(set '_segments (parse (trim (lookup "REQUEST_URI" _cgi-env) "/") "/"))


(define (method)
  _method)


(define (get?)
  (= _method "get"))


(define (post?)
  (= _method "post"))


(define (get key)
  (if key
    (lookup key _get)
    _get))


(define (post key)
  (if key
    (lookup key _post)
    _post))


(define (cookies key)
  (if key
    (lookup key _cookies)
    _cookies))


(define (cookie? key)
  (lookup key _cookies))


(set '_current-segment -1)
(define (segment)
  (unless (>= _current-segment (- (length _segments) 1))
    (nth (if (args) (args 0) (inc '_current-segment)) _segments)
    nil))


(define (segments)
  _segments)


(set 'path (lookup "REQUEST_URI" _cgi-env))
(set 'domain (lookup "HTTP_HOST" _cgi-env))


(context MAIN)

